#!/bin/bash
#
# Copyright (c) 2019-2022 Oracle and/or its affiliates. All rights reserved.
# Licensed under the GNU General Public License Version 3 as shown at https://www.gnu.org/licenses/gpl-3.0.txt.

set -o nounset
set -o pipefail
set -o errtrace

# variables
TO_REGISTRY=
FROM_REGISTRY=container-registry.oracle.com/olcne
OLCNE_DIRECTORY=/etc/olcne
USELOCAL=0
MODULE_VERSION=
MODULE_NAME=

function exit_trap {
    local rc=$?
    local lc="$BASH_COMMAND"

    if [[ $rc -ne 0 ]]; then
        echo "Command [$lc] exited with code [$rc]"
    fi
}

trap exit_trap EXIT

function pull {
    image=$1

    echo ">> Pulling image: ${FROM_REGISTRY}/${image}"
    podman pull "${FROM_REGISTRY}/${image}"
}

function tag {
    image=$1

    echo ">> Tagging image: ${TO_REGISTRY}/${image}"
    podman tag "${FROM_REGISTRY}/${image}" "${TO_REGISTRY}/${image}"
}

function push {
    image=$1

    echo ">> Pushing image: ${TO_REGISTRY}/${image}"
    podman push "${TO_REGISTRY}/${image}"
}

function getImagesFromModule {
    moduleFile=$1
    moduleVersion=$2

    grep 'versions:' "${moduleFile}" -A1000 | grep END_VERSION_BLOCK -B 1000  | yq r - versions.["${moduleVersion}"].container-images
}

function getKubernetesImages {
    kubernetesVersion=$1

    FILES=$(find ${OLCNE_DIRECTORY} -name kubernetes.yaml)

    for filename in $FILES; do
        VERS=$(getImagesFromModule "${filename}" "${kubernetesVersion}" | awk 'NF && !seen[$0]++' -)
    done

    echo "$VERS"

    # TODO: We need to handle these images inside the module definition but not pull them down on every node
    function 1.17.9_extra_images {
        echo "kubernetes-dashboard: v1.10.1-2"
        echo "flannel: v0.12.0"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0"
    }
    function 1.18.10_extra_images {
        echo "kubernetes-dashboard: v2.0.3"
        echo "flannel: v0.12.0"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0"
    }
    function 1.18.18_extra_images {
        echo "kubernetes-dashboard: v2.0.3-1"
        echo "flannel: v0.12.0-1"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0-1"
    }
    function 1.20.6_extra_images {
        echo "kubernetes-dashboard: v2.1.0"
        echo "flannel: v0.13.0"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0-1"
    }
    function 1.20.11_extra_images {
        echo "kubernetes-dashboard: v2.1.0"
        echo "flannel: v0.13.0"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0-1"
    }
    function 1.21.6_extra_images {
        echo "kubernetes-dashboard: v2.3.1"
        echo "flannel: v0.14.1"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0-1"
    }
    function 1.22.8_extra_images {
        echo "kubernetes-dashboard: v2.3.1"
        echo "flannel: v0.14.1"
        echo "nginx: 1.17.7"
        echo "externalip-webhook: v1.0.0-1"
    }
    case "${kubernetesVersion}" in
        1.17.9)
            1.17.9_extra_images
        ;;

        1.18.10)
            1.18.10_extra_images
        ;;

        1.18.18)
            1.18.18_extra_images
        ;;

        1.20.6)
            1.20.6_extra_images
        ;;
        1.20.11)
            1.20.11_extra_images
        ;;
        1.21.6)
            1.21.6_extra_images
        ;;
        1.22.8)
            1.22.8_extra_images
        ;;

        # All images
        *)
            1.17.9_extra_images
            1.18.10_extra_images
            1.18.18_extra_images
            1.20.6_extra_images
            1.20.11_extra_images
            1.21.6_extra_images
            1.22.8_extra_images
        ;;
    esac
}

function getIstioImages {
    istioVersion=$1

    FILES=$(find ${OLCNE_DIRECTORY} -name istio.yaml)

    for filename in $FILES; do
        VERS=$(getImagesFromModule "${filename}" "${istioVersion}" | awk 'NF && !seen[$0]++' -)
    done

    echo "$VERS"
}

function getPrometheusImages {
    prometheusVersion=$1

    FILES=$(find ${OLCNE_DIRECTORY} -name prometheus.yaml)

    for filename in $FILES; do
        VERS=$(getImagesFromModule "${filename}" "${prometheusVersion}" | awk 'NF && !seen[$0]++' -)
    done

    echo "$VERS"
}

function getOLMImages {
    olmVersion=$1

    FILES=$(find ${OLCNE_DIRECTORY} -name olm.yaml)

    for filename in $FILES; do
        VERS=$(getImagesFromModule "${filename}" "${olmVersion}" | awk 'NF && !seen[$0]++' -)
    done

    echo "$VERS"
}

function getImages {
    all_images_string="*"

    module="${1:-ALL}"
    moduleVersion="${2:-${all_images_string}}"

    case ${module} in
        kubernetes)
            getKubernetesImages "${moduleVersion}"
            ;;
        istio)
            getIstioImages "${moduleVersion}"
            ;;
        prometheus)
            getPrometheusImages "${moduleVersion}"
            ;;
        operator-lifecycle-manager)
            getOLMImages "${moduleVersion}"
            ;;
        ALL)
            getKubernetesImages "${moduleVersion}"
            getIstioImages "${moduleVersion}"
            getPrometheusImages "${moduleVersion}"
            getOLMImages "${moduleVersion}"
            ;;
    esac
}

function check {
    echo "Checking if podman is installed ..."
    if ! podman --help > /dev/null; then
        echo "[ERROR] podman is not install ... please install podman"
        exit 1
    fi

    echo "Checking if yq is installed ..."
    if ! yq --help > /dev/null; then
        echo "[ERROR] yq is not install ... please install yq"
        exit 1
    fi
}

function usage {
    echo "This script is to help pulling container images from default container-registry.oracle.com to a local repo" >&2
    echo "usage: " >&2
    echo "  $0 --to registry [--local --from registry --module <MODULE NAME> --version <MODULE VERSION>]" >&2
    exit 1
}

function main {
    # Check the system to fail early
    check

    # Load all local images
    if [ "$USELOCAL" -eq 1 ]; then
        if ls /usr/local/share/kubeadm/*.tar &> /dev/null; then
            find /usr/local/share/kubeadm/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
        if ls /usr/local/share/olcne/*.tar &> /dev/null; then
            find /usr/local/share/olcne/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
    fi

    # echo $imageInfo
    images=$(getImages "${MODULE_NAME}" "${MODULE_VERSION}")

    # Loop through all the images
    while IFS= read -r imageInfo ; do
        # Remove space to from the image name e.g. 'proxyv2: 1.7.3'
        image="${imageInfo// /}"
        success=false
        for i in {1..10}; do
            # Only pull the image if we are not looking at local images
            if [ "$USELOCAL" -eq 0 ]; then
                pull "$image"
                if [[ $? -ne 0 ]]; then
                    sleep 30
                    continue
                fi
            fi

            tag "$image"
            if [[ $? -ne 0 ]]; then
                sleep 30
                continue
            fi

            push "$image"
            if [[ $? -ne 0 ]]; then
                sleep 30
                continue
            fi

            success=true
            break
        done

        if [[ "${success}" == "false" ]]; then
            echo "[ERROR] Failed to manage image [${image}]"
            exit 1
        fi
    done <<< "${images}"

    echo "[SUCCESS] All images pushed to [${TO_REGISTRY}]"
}

# MAIN
while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in
        --to )
            TO_REGISTRY=$2
            if [ -z "${TO_REGISTRY}" ]; then
                echo "[ERROR] Please provide a valid local registry location"
                exit 1
            fi
            shift # shift past argument
            shift # shift past value
        ;;
        --from )
            FROM_REGISTRY="${2:-${REGISTRY}}"
            shift # shift past argument
            shift # shift past value
        ;;
        --module )
            MODULE_NAME="${2}"
            shift # shift past argument
            shift # shift past value
        ;;
        --version )
            MODULE_VERSION="${2}"
            shift # shift past argument
            shift # shift past value
        ;;
        --local )
            USELOCAL=1
            shift # shift past argument
        ;;
        --module-dir )
            OLCNE_DIRECTORY="${2}"
            shift # shift past argument
            shift # shift past value
        ;;
        -h | --help )
            usage
            exit
        ;;
        *) # unknown option
            echo "[ERROR] unknown argument: ${key}"
            usage
            exit 1
        ;;
    esac
done

main
